/**
 * 
 */
package net.bigeon.gclc.process;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.junit.Test;

import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.process.mocks.ForkTaskMock;

/**
 * @author Emmanuel Bigeon
 *
 */
public class CommandForegroundTest {

    /**
     * Test method for {@link net.bigeon.gclc.process.CommandForeground#CommandForeground(java.lang.String, net.bigeon.gclc.process.TaskPool)}.
     */
    @Test
    public void testCommandForeground() {
        final CommandForeground pl = new CommandForeground("fg", new TaskPool());
        assertEquals("Command name should be kept as specified", "fg",
                pl.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.process.CommandForeground#doExecute(net.bigeon.gclc.manager.ConsoleOutput, net.bigeon.gclc.manager.ConsoleInput, net.bigeon.gclc.command.CommandParameters)}.
     * 
     * @throws CommandRunException if a valid command failed */
    @Test
    public void testDoExecuteConsoleOutputConsoleInputCommandParameters()
            throws CommandRunException {
        final TaskPool pool = new TaskPool(false);
        final CommandForeground pl = new CommandForeground("fg", pool);

        try {
            pl.execute(null, null, "-delai", "1");
            fail("PID should be mandatory");
        } catch (final CommandRunException e) {
            assertEquals("No pid specified is a usage error",
                    CommandRunExceptionType.USAGE, e.getType());
        }
        try {
            pl.execute(null, null, "-pid", "invalid", "-delai", "1");
            fail("PID should be a number");
        } catch (final CommandRunException e) {
            assertEquals("Invalid pid specified is a usage error",
                    CommandRunExceptionType.USAGE, e.getType());
        }
        try {
            pl.execute(null, null, "-pid", "2", "-delai", "1");
            fail("PID should exist");
        } catch (final CommandRunException e) {
            assertEquals("Inexistent pid specified is a run error",
                    CommandRunExceptionType.EXECUTION, e.getType());
        }

        final ForkTaskMock cmd = new ForkTaskMock();
        final int id = pool.add(cmd);
        pl.execute(null, null, "-pid", Integer.toString(id), "-delai", "1");

        try {
            pl.execute(null, null, "-pid", Integer.toString(id), "-delai", "-1");
            fail("delai should be a posistive number");
        } catch (final CommandRunException e) {
            assertEquals("Negative delai specified is a usage error",
                    CommandRunExceptionType.USAGE, e.getType());
        }

        assert cmd.getRunCall() == 0;
    }

    /**
     * Test method for {@link net.bigeon.gclc.process.CommandForeground#tip()}.
     */
    @Test
    public void testTip() {
        final CommandForeground pl = new CommandForeground("fg", new TaskPool());
        assertNotNull("Command tip should be defined", pl.tip());
        assertNotNull("Command usage should be defined", pl.usagePattern());
        assertNotNull("Command usage should be defined", pl.usageDetail());
    }

}
